/***************************************************************************
 *   Copyright (C) 2007 by Sébastien Laoût                                 *
 *   slaout@linux62.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "progressbar.h"

#include <qpainter.h>
#include <qpixmap.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <kglobalsettings.h>
#include <klocale.h>
#include <qtooltip.h>

ProgressBar::ProgressBar(QWidget *parent, const char *name)
 : QWidget(parent, name, /*Qt::WStaticContents | */Qt::WNoAutoErase),
   m_total(0), m_current(0), m_underMouse(false), m_mousePressed(false)
{
}

ProgressBar::~ProgressBar()
{
}

void ProgressBar::setTotal(int total)
{
	if (total != m_total) {
		m_total = total;
		if (m_current > m_total)
			m_current = m_total;
		update();
	}
}

void ProgressBar::setCurrent(int current)
{
	if (current != m_current) {
		m_current = current;
		if (m_current > m_total)
			m_current = m_total;
		update();
	}
}

void ProgressBar::setLeftText(const QString &leftText)
{
	if (m_leftText != leftText) {
		m_leftText = leftText;
		update();
	}
}

void ProgressBar::setRightText(const QString &rightText)
{
	if (m_rightText != rightText) {
		m_rightText = rightText;
		update();
	}
}

void ProgressBar::paintEvent(QPaintEvent */*event*/)
{
	const int BORDER_DARK = 140;
	const int DARKING     = (height() > 16 ? 190 : 120);
	const int LIGHTING    = 120;

	// Begin drawing:
	QPixmap buffer(width(), height());
	QPainter painter(&buffer);

	QColor secondBaseColor = KGlobalSettings::baseColor().dark(DARKING);

	QColor barColor = KGlobalSettings::highlightColor();
	if (m_underMouse) {
		barColor = barColor.light(110);
		secondBaseColor = KGlobalSettings::baseColor().dark(100 + (DARKING - 100) / 2);
	}

	// Draw the surrounding frame:
	QColor borderColor = barColor;
	int h, s, v;
	borderColor.getHsv(&h, &s, &v);
	borderColor = QColor(h, 255, v, QColor::Hsv).dark(BORDER_DARK);
	painter.setPen(borderColor);
	painter.drawRect(0, 0, width(), height());

	// Draw the background:
	KPixmap gradient;
	gradient.resize(width() - 2, height() - 2);
	KPixmapEffect::gradient(gradient, KGlobalSettings::baseColor(), secondBaseColor, KPixmapEffect::VerticalGradient);
	painter.drawPixmap(1, 1, gradient);

	// Draw the background light inner-border:
	QColor topColor    = KGlobalSettings::baseColor();
	QColor bottomColor = secondBaseColor.light(LIGHTING);
	painter.setPen(topColor);
	painter.drawLine(1, 1, width() - 2, 1);
	painter.setPen(bottomColor);
	painter.drawLine(1, height() - 2, width() - 2, height() - 2);
	gradient.resize(1, height() - 4);
	KPixmapEffect::gradient(gradient, topColor, bottomColor, KPixmapEffect::VerticalGradient);
	painter.drawPixmap(1, 2, gradient);
	painter.drawPixmap(width() - 2, 2, gradient);

	int barWidth = (m_total == 0 ? 0 : (width() - 2) * m_current / m_total);

	if (barWidth > 0) {
		// Draw the progress fill:
		gradient.resize(barWidth, height() - 2);
		KPixmapEffect::gradient(gradient, barColor, barColor.dark(DARKING), KPixmapEffect::VerticalGradient);
		painter.drawPixmap(1, 1, gradient);

		// Draw the progress light inner-border:
		topColor    = barColor.light(LIGHTING);
		bottomColor = barColor.dark(DARKING).light(LIGHTING);
		painter.setPen(topColor);
		painter.drawLine(1, 1, barWidth, 1);
		painter.setPen(bottomColor);
		painter.drawLine(1, height() - 2, barWidth, height() - 2);
		gradient.resize(1, height() - 4);
		KPixmapEffect::gradient(gradient, topColor, bottomColor, KPixmapEffect::VerticalGradient);
		painter.drawPixmap(1, 2, gradient);
		if (barWidth == width() - 2)
			painter.drawPixmap(width() - 2, 2, gradient);
	}

	// Draw the left & right texts if the size is sufficient:
	if (height() >= 16) {
		int margin = 4;
		QFont font = painter.font();
		font.setPixelSize(height() - 2 * margin);
		font.setBold(true);
		painter.setFont(font);

		painter.setClipRect(1 + barWidth, 0, width() - 1 - barWidth, height());
		painter.setPen(KGlobalSettings::textColor());
		painter.drawText(QRect(margin, margin, width() - 2 * margin, height() - 2 * margin), Qt::AlignLeft  | Qt::AlignVCenter, m_leftText);
		painter.drawText(QRect(margin, margin, width() - 2 * margin, height() - 2 * margin), Qt::AlignRight | Qt::AlignVCenter, m_rightText);

		painter.setClipRect(0, 0, 1 + barWidth, height());
		painter.setPen(KGlobalSettings::highlightedTextColor());
		painter.drawText(QRect(margin, margin, width() - 2 * margin, height() - 2 * margin), Qt::AlignLeft  | Qt::AlignVCenter, m_leftText);
		painter.drawText(QRect(margin, margin, width() - 2 * margin, height() - 2 * margin), Qt::AlignRight | Qt::AlignVCenter, m_rightText);
	}

	// End drawing:
	painter.end();
	QPainter painterThis(this);
	painterThis.drawPixmap(0, 0, buffer);
	painterThis.end();
}

void ProgressBar::mousePressEvent(QMouseEvent *event)
{
	m_mousePressed = true;
	changePosition(event);
}

void ProgressBar::mouseReleaseEvent(QMouseEvent *event)
{
	m_mousePressed = false;
}

void ProgressBar::removeHighlighting()
{
	if (m_underMouse) {
		m_underMouse = false;
		update();
	}
}

void ProgressBar::mouseMoveEvent(QMouseEvent *event)
{
	// If the progressbar highlighting was disabled (because cursor was hidden in full screen), re-highlight it again:
	if (m_underMouse == false) {
		m_underMouse = true;
		update();
	}

//	// This mouseEvent is received only when dragging, so we change the progress position:
	if (m_mousePressed)
		changePosition(event);

	QWidget::mouseMoveEvent(event);
}

void ProgressBar::changePosition(QMouseEvent *event)
{
	int x = event->pos().x();
	if (x > 0 && x < width() - 1)
		emit changePosition((x - 1) * m_total / (width() - 2));
}

void ProgressBar::enterEvent(QEvent *event)
{
	// This is only to get MouseMoveEvent is FullScreen mode, to re-show mouse cursor (with an event filter in CoverDisplay)!
	setMouseTracking(true);

	// Highlight the progressbar on mouse hover:
	m_underMouse = true;
	update();

	QWidget::enterEvent(event);
}

void ProgressBar::leaveEvent(QEvent *event)
{
	setMouseTracking(false);

	// Disable the progressbar highlighting because mouse is now out:
	m_underMouse = false;
	update();

	QWidget::leaveEvent(event);
}

#include "progressbar.moc"
